"""
# The authors stated
# that the network being used was Keras MNIST CNN.
# The structure of this network can be found in the following
# link: https: // keras.io/examples/mnist_cnn/
# Function to apply Glorot Initialization on the netowrk
"""
from torch import nn
import torch


def weight_init(m):
    if isinstance(m, nn.Conv2d) or isinstance(m, nn.Linear):
        torch.nn.init.xavier_uniform_(
            m.weight, gain=nn.init.calculate_gain('relu'))
        torch.nn.init.zeros_(m.bias)


class MNIST_CNN(nn.Module):
    def __init__(self,):
        super(MNIST_CNN, self).__init__()
        self.convs = nn.Sequential(
            nn.Conv2d(1, 32, 3),
            nn.ReLU(),
            nn.Conv2d(32, 64, 3),
            nn.ReLU(),
            nn.MaxPool2d(2),
            nn.Dropout(0.25)
        )
        self.convs.apply(weight_init)
        self.fcs = nn.Sequential(
            nn.Linear(12*12*64, 128),
            nn.ReLU(),
            nn.Dropout(0.5),
            nn.Linear(128, 10),
        )
        self.fcs.apply(weight_init)

    def forward(self, x):
        out = x
        out = self.convs(out)
        out = out.view(-1, 12*12*64)
        out = self.fcs(out)
        return out



from enum import Enum
import os
from os.path import join
import shutil
import datetime
from pprint import pprint
import sys
from .ds import ds
import torch
from torch.utils.tensorboard import SummaryWriter

sys.path.insert(0, '..')


def shape(*args):
    for idx, item in enumerate(args):
        print(idx, getattr(item, 'shape', 'No Shape Defined'))


class AL_ENVS(Enum):
    COLAB = 0
    LOCAL = 1


conf = {}


def datasets(func, transforms=None):
    return getattr(conf['AL_DATASETS'], func, lambda x: "Method not fuond!")(transforms)


def set_tb_log_dir_with_history(log_dir_path, datetime_format="%Y%m%d-%H%M%S"):
    """
        For each new experiment, create a new directory with current
        datetime to store tensorboard logs.
    """
    log_dir = os.path.join(
        log_dir_path, datetime.datetime.now().strftime(datetime_format))

    if os.path.isdir(log_dir):
        raise "Directory name clashes with old logs."
    else:
        print(
            'Info: New directory will be created and setup for tensorboard.')
        os.makedirs(log_dir, exist_ok=True)


def set_tb_log_dir_with_no_history(log_dir_path):
    """
        For each new experiment, discard old logs, and use the same path again.
    """
    log_dir = os.path.join(
        log_dir_path)

    print('Info: Removing old logs, and setup log directory for tensorboard.')
    rm_logs(log_dir)
    os.makedirs(log_dir, exist_ok=True)


def set_defaults(settings={}, print_settings=True):
    """
        Set varilable in the order of presedence.
        1. There are defaults for everything.
        2. settings dict override defaults.
        3. Environment Overrides settings.

    """
    global conf
    print("Info: Setting up defalut log directory for tensorboard.")
    AL_EXPERIMENT = os.environ.get(
        'AL_EXPERIMENT', settings.get('AL_EXPERIMENT', 'default-experiment'))
    AL_ENV = os.environ.get(
        'AL_ENV', settings.get('AL_ENV', None))
    AL_BASE_PATH = os.environ.get(
        'AL_BASE_PATH', settings.get('AL_BASE_PATH', './'))
    AL_DEFAULT_LOG_DIR = os.environ.get(
        'AL_DEFAULT_LOG_DIR', settings.get('AL_DEFAULT_LOG_DIR', join(AL_BASE_PATH, AL_EXPERIMENT, 'logs')))
    AL_DEFAULT_DATA_DIR = os.environ.get(
        'AL_DEFAULT_DATA_DIR', settings.get('AL_DEFAULT_DATA_DIR', join(AL_BASE_PATH, AL_EXPERIMENT, 'data')))
    AL_EXPERIMENT_DIR = os.environ.get(
        'AL_EXPERIMENT_DIR', settings.get(
            'AL_EXPERIMENT_DIR', join(AL_BASE_PATH, AL_EXPERIMENT)))

    if AL_ENV is None:
        try:
            import google.colab
            AL_ENV = AL_ENVS.COLAB
        except:
            AL_ENV = AL_ENVS.LOCAL
        assert isinstance(
            AL_ENV, AL_ENVS), "Environment must be of type AL_ENVS"

    AL_LOG_MODE = os.environ.get('AL_LOG_MODE', 'history')
    if AL_LOG_MODE == 'history':
        set_tb_log_dir_with_history(AL_DEFAULT_LOG_DIR)
    else:
        set_tb_log_dir_with_no_history(AL_DEFAULT_LOG_DIR)

    _ = {
        'AL_ENV': AL_ENV,
        'AL_DEFAULT_LOG_DIR': AL_DEFAULT_LOG_DIR,
        'AL_DEFAULT_DATA_DIR': AL_DEFAULT_DATA_DIR,
        'AL_BASE_PATH': AL_BASE_PATH,
        'AL_LOG_MODE': AL_LOG_MODE,
        'AL_ENV': AL_ENV,
        'AL_DATASETS': ds(AL_DEFAULT_DATA_DIR),

        'device': "cuda" if torch.cuda.is_available() else "cpu",
        'tensorboard': SummaryWriter(log_dir=AL_DEFAULT_LOG_DIR)
    }

    conf = _
    if print_settings:
        pprint(_)
    return _


def is_colab():
    """
        A function for testing weather the current environment is colab
        ----
        Parameters
            - None
        Return
            - Boolean
    """
    return conf['AL_ENV'] == AL_ENVS.COLAB


def rm_logs(log_path):
    if os.path.isdir(log_path):
        shutil.rmtree(log_path)
        return True
    return False


def load_model(experiment_path):
    """
        Load model from default model location with name.
        ----
        Parameters
            - str: experiment_name
        Return
            - nn.Module: net
            - tuple    : (train_set, test_set)
    """
    pass


def store_model(net, dataset_state):
    """
        Store the given model at default model location.
        ----
        Parameters
            - None
        Return
            - Boolean
    """
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

""" Dataset
While there are so many good packages, the purposes of this doc
is to focus on other changes and leave dataset loading stuff here
and then forget it.

This file should support all datasets we need to use again and again.
It is not mandatory to use it, i hope it will save time to use this.

Note that, these methods should make the transition between colab and 
local envs easy for user.

There is no need to make a generic functinos, just ones for our use case.

"""
import torch
from torch.utils.data import DataLoader
from torchvision.datasets import MNIST, CIFAR10
from os.path import join


class ds():
    def __init__(self, base_path):
        super().__init__()
        assert len(base_path) > 0, "Base path should not be empty."
        self.base_path = base_path

    def mnist(self, transforms, train_prefix='mnist/train', test_prefix='mnist/test'):
        _path_train = join(self.base_path, train_prefix)
        _path_test = join(self.base_path, train_prefix)
        train_set = DataLoader(
            MNIST(_path_train, download=True, train=True, transform=transforms))
        test_set = DataLoader(
            MNIST(_path_test, download=True, train=False, transform=transforms))

        X_train = train_set.dataset.data.to(
            dtype=torch.float32).view(-1, 1, 28, 28)
        y_train = train_set.dataset.targets.to(dtype=torch.long)

        X_test = test_set.dataset.data.to(
            dtype=torch.float32).view(-1, 1, 28, 28)
        y_test = test_set.dataset.targets.to(dtype=torch.long)

        return X_train, y_train, X_test, y_test

    def cifar10(self, train_prefix='cifar10/train', test_prefix='cifar10/test'):
        _path_train = join(self.base_path, train_prefix)
        _path_test = join(self.base_path, train_prefix)
        train_set = DataLoader(
            CIFAR10(_path_train, download=True, train=True, transform=transforms))
        test_set = DataLoader(
            CIFAR10(_path_test, download=True, train=False, transform=transforms))

        return None
        # X_train = train_set.dataset.data.to(
        #     dtype=torch.float32).view(-1, 1, 28, 28)
        # y_train = train_set.dataset.targets.to(dtype=torch.long)

        # X_test = test_set.dataset.data.to(
        #     dtype=torch.float32).view(-1, 1, 28, 28)
        # y_test = test_set.dataset.targets.to(dtype=torch.long)

        # return X_train, X_test, y_train, y_test
    def dibetes():
        pass



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="active_learning_img_augmentation_utils",
    version="1.0.1",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=[
        'torch',
    ],
)
